/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: AbstractMnemonic.java
 * EMail: W.Klaas@gmx.de
 * Created: 25.11.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps.mnemonic;

import java.util.EnumSet;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * @author wklaa_000
 *
 */
public abstract class AbstractMnemonic implements Mnemonic {

  private String srcLine;
  private String name;
  private String argument;
  private int lineNumber;

  public static String getName(String line) {
    String srcLine = line.trim();
    if (srcLine.indexOf(" ") >= 0) {
      String[] values = srcLine.split(" ");
      return values[0].toUpperCase();
    }
    return srcLine.toUpperCase();
  }

  AbstractMnemonic(String line) throws SyntaxError {
    this.srcLine = line.trim();
    this.name = srcLine;
    int pos = srcLine.indexOf(" ");
    if (pos >= 0) {
      this.name = srcLine.substring(0, pos).toUpperCase();
      this.argument = srcLine.substring(pos).trim();
    }
  }

  /**
   * default implementation for checking no argument mnemonics
   */
  @Override
  public void checkArgument() throws SyntaxError {
    if (StringUtils.isNotEmpty(argument)) {
      throw new SyntaxError(getLineNumber(), String.format("%s has no argument.", this.getClass().getSimpleName()));
    }
  };

  @Override
  public String toString() {
    return String.format("%s %s", name, StringUtils.isEmpty(argument) ? "" : argument);
  }

  /**
   * @return the name
   */
  @Override
  public String getName() {
    return name;
  }

  /**
   * @return the argument
   */
  @Override
  public String getArgument() {
    return argument;
  }

  @Override
  public void setArgument(String argument) {
    this.argument = argument;
  }

  public int getArgumentAsNumber() throws IllegalArgument {
    return getArgumentAsNumber(getArgument());
  };

  public int getArgumentAsNumber(String arg) throws IllegalArgument {
    int value = 0;
    try {
      String argument = arg.toLowerCase();
      if (argument.startsWith("0b")) {
        value = Integer.parseInt(argument.substring(2), 2);
      } else {
        value = Integer.decode(argument);
      }
    } catch (Exception e) {
      throw new IllegalArgument(getLineNumber(),
          String.format("argument %s is not valid for %s.", arg, this.getClass().getSimpleName()));
    }
    return value;
  };

  public abstract int getByte();

  @Override
  public boolean isUsingPage() {
    return false;
  }

  @Override
  public int getLineNumber() {
    return lineNumber;
  }

  @Override
  public void setLineNumber(int lineNumber) {
    this.lineNumber = lineNumber;
  }

  @Override
  public boolean hasArgument() {
    return false;
  }

  @Override
  public boolean isLabel() {
    return false;
  }

  @Override
  public Set<HARDWARE> allowedHardware() {
    return EnumSet.allOf(HARDWARE.class);
  }
}
